// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_common.h $
// $Rev: 22965 $
// $Date: 2013-08-26 15:03:51 -0500 (Mon, 26 Aug 2013) $

// INtime 4 driver

#ifndef __OS_COMMON_H__
#define __OS_COMMON_H__

#include <limits.h>

#include "rt.h"



// #defines *******************************************************************

#define	OS_MSG_SIZE_COMMON			8	// fields before the union
#define	OS_MSG_SIZE_CLOSE			16
#define	OS_MSG_SIZE_EXIT			12
#define	OS_MSG_SIZE_IOCTL_LG		36
#define	OS_MSG_SIZE_IOCTL_SM_MAX	128
#define	OS_MSG_SIZE_IOCTL_SM_MIN	20
#define	OS_MSG_SIZE_OPEN			16
#define	OS_MSG_SIZE_READ			28
#define	OS_MSG_SIZE_WRITE			28

#define	OS_IOCTL_DIR_READ			GSC_FIELD_ENCODE(1,26,25)	// app's view
#define	OS_IOCTL_DIR_WRITE			GSC_FIELD_ENCODE(2,26,25)	// app's view
#define	OS_IOCTL_DIR_DECODE(c)		((c) & (OS_IOCTL_DIR_READ | OS_IOCTL_DIR_WRITE))

#define	OS_IOCTL_TYPE_ENCODE(v)		GSC_FIELD_ENCODE((v), 7, 0)
#define	OS_IOCTL_TYPE_DECODE(c)		GSC_FIELD_DECODE((c), 7, 0)

#define	OS_IOCTL_INDEX_ENCODE(v)	GSC_FIELD_ENCODE((v),15, 8)
#define	OS_IOCTL_INDEX_DECODE(c)	GSC_FIELD_DECODE((c),15, 8)

#define	OS_IOCTL_SIZE_ENCODE(v)		GSC_FIELD_ENCODE((v),24,16)
#define	OS_IOCTL_SIZE_DECODE(c)		GSC_FIELD_DECODE((c),24,16)

#define	OS_IOCTL_R(i,s,t)			( OS_IOCTL_DIR_READ					\
									| OS_IOCTL_TYPE_ENCODE(GSC_IOCTL)	\
									| OS_IOCTL_INDEX_ENCODE((i))		\
									| OS_IOCTL_SIZE_ENCODE(s))

#define	OS_IOCTL_RW(i,s,t)			( OS_IOCTL_DIR_READ					\
									| OS_IOCTL_DIR_WRITE				\
									| OS_IOCTL_TYPE_ENCODE(GSC_IOCTL)	\
									| OS_IOCTL_INDEX_ENCODE((i))		\
									| OS_IOCTL_SIZE_ENCODE(s))

#define	OS_IOCTL_W(i,s,t)			( OS_IOCTL_DIR_WRITE				\
									| OS_IOCTL_TYPE_ENCODE(GSC_IOCTL)	\
									| OS_IOCTL_INDEX_ENCODE((i))		\
									| OS_IOCTL_SIZE_ENCODE(s))

#ifndef ERESTARTSYS
#define	ERESTARTSYS					512
#endif



// data types *****************************************************************

#if (UINT_MAX == 4294967295U)

typedef signed int		s32;
typedef unsigned int	u32;

#else

	#error	The data types s32 and u32 are undefined.

#endif

#if (USHRT_MAX == 65535)

typedef signed short	s16;
typedef unsigned short	u16;

#else

	#error	The data types s16 and u16 are undefined.

#endif

#if (UCHAR_MAX == 255)

typedef signed char		s8;
typedef unsigned char	u8;

#else

	#error	The data types s8 and u8 are undefined.

#endif

typedef enum
{
	OS_MSG_STS_OK,
	OS_MSG_STS_MSG_SIZE,
	OS_MSG_STS_MSG_TYPE,
	OS_MSG_STS_MEM_MAP,
	OS_MSG_STS_THREAD
} os_msg_sts_t;

typedef enum
{
	OS_MSG_TYPE_OPEN	= 1,
	OS_MSG_TYPE_CLOSE,
	OS_MSG_TYPE_IOCTL_SM,	// <= 108 bytes
	OS_MSG_TYPE_IOCTL_LG,	// > 108 bytes
	OS_MSG_TYPE_READ,
	OS_MSG_TYPE_WRITE,
	OS_MSG_TYPE_EXIT
} os_msg_type_t;

typedef struct
{
	s32		results;
	s32		err_no;
} os_msg_close_t;

typedef struct
{
	u8		flags;
	u8		reserved_1;
	u8		reserved_2;
	u8		reserved_3;
} os_msg_exit_t;

typedef struct
{
	s32		results;
	s32		err_no;
	s32		cmd;
	u8		buf[104];
} os_msg_ioctl_small_t;

typedef struct
{
	s32		results;
	s32		err_no;
	s32		cmd;
	u32		mem;		// memory handle
	u32		offset;		// offset within memory area
} os_msg_ioctl_large_t;

typedef struct
{
	s32		results;
	s32		err_no;
} os_msg_open_t;

typedef struct
{
	s32		results;
	s32		err_no;
	s32		count;
	u32		mem;		// memory handle
	u32		offset;		// offset within memory area
} os_msg_read_t;

typedef struct
{
	s32		results;
	s32		err_no;
	s32		count;
	u32		mem;		// memory handle
	u32		offset;		// offset within memory area
} os_msg_write_t;

typedef struct				// 128 bytes in size
{
	u8			status;		// of os_msg_sts_t content
	u8			msg_type;	// os_msg_type_t
	u8			size;
	u8			reserved_2;
	u32			mbox;		// RTHANDLE	send responses here

	union
	{
		os_msg_open_t			open;
		os_msg_close_t			close;
		os_msg_ioctl_small_t	ioctl_sm;
		os_msg_ioctl_large_t	ioctl_lg;
		os_msg_read_t			read;
		os_msg_write_t			write;
		os_msg_exit_t			exit;
	} u;
} os_msg_t;



#endif
