// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_event.c $
// $Rev: 26098 $
// $Date: 2014-05-16 10:50:37 -0500 (Fri, 16 May 2014) $

// INtime 4 driver

#include "main.h"



//*****************************************************************************
void os_event_cleanup(os_event_t* evnt)
{
	if (evnt == NULL)
	{
		// The pointer is NULL.
	}
	else if (evnt->key != evnt)
	{
		// The semaphore was never created.
		memset(evnt, 0, sizeof(os_event_t));
	}
	else
	{
		// Delete the semaphore.
		knDeleteRtSemaphore(evnt->handle);
		memset(evnt, 0, sizeof(os_event_t));
	}
}



//*****************************************************************************
void os_event_init(os_event_t* evnt)
{
	if (evnt == NULL)
	{
		// The pointer is NULL.
	}
	else if (evnt->key == evnt)
	{
		// The semaphore has already been created.
	}
	else
	{
		// Create a new, one unit semaphore.
		evnt->handle	= knCreateRtSemaphore((KNPTR) &evnt->sem_data, KN_FIFO_QUEUING | KN_ZERO_UNITS);
		evnt->key		= (void*) evnt;
	}
}



//*****************************************************************************
void os_event_resume(os_event_t* evnt)
{
	if (evnt == NULL)
	{
		// The pointer is NULL.
	}
	else if (evnt->key != evnt)
	{
		// The semaphore was never created.
	}
	else
	{
		// Release access to the semaphore.
		knReleaseRtSemaphore(evnt->handle);
	}
}



//*****************************************************************************
void os_event_wait(os_event_t* evnt, os_time_tick_t timeout)
{
	if (evnt == NULL)
	{
		// The pointer is NULL.
	}
	else if (evnt->key != evnt)
	{
		// The semaphore was never created.
	}
	else
	{
		// Gain access to the semaphore.
		knWaitForRtSemaphore(evnt->handle, (DWORD) timeout);
	}
}


