// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/lib/intime/os_close.c $
// $Rev: 26099 $
// $Date: 2014-05-16 10:51:54 -0500 (Fri, 16 May 2014) $

// INtime 4 library

#include "main.h"



//*****************************************************************************
// returns err_no: 0 = ok, !=0 = error value
int os_close(int fd)
{
	int			err_no	= 0;
	os_file_t*	fp		= (os_file_t*) fd;
	os_mbox_t*	mbox	= NULL;
	os_msg_t*	msg		= NULL;
	int			sem		= BAD_RTHANDLE;
	WORD		size;
	DWORD		sts;

	for (;;)
	{
		if ((fd <= 0) || (fp->val_ptr != fp))
		{
			err_no	= EINVAL;
			break;
		}

		// Acquire access to the structure.
		sem	= os_sem_lock(&fp->sem);

		if (sem)
		{
			errno	= sem;
			break;
		}

		// Acquire a mailbox for the receive message.
		err_no	= os_mbox_alloc(&mbox);

		if (err_no)
			break;

		// Allocate a message structure.
		err_no	= os_msg_alloc(&msg);

		if (err_no)
			break;

		// Initialize and send the message.
		memset(msg, 0, sizeof(os_msg_t));
		msg->status		= OS_MSG_STS_OK;
		msg->msg_type	= OS_MSG_TYPE_CLOSE;
		msg->size		= OS_MSG_SIZE_CLOSE;
		msg->mbox		= mbox->mbox;
		sts				= SendRtData(fp->dev_mbox, msg, OS_MSG_SIZE_CLOSE);

		if (sts == FALSE)
		{
			err_no	= GetLastRtError();
			break;
		}

		// Get and process the response.
		size	= ReceiveRtData(mbox->mbox, msg, WAIT_FOREVER);

		if ((size != OS_MSG_SIZE_CLOSE) || (size != msg->size))
		{
			err_no	= EIO;
			break;
		}

		switch (msg->status)
		{
			default:

				err_no	= EBADMSG;
				break;

			case OS_MSG_STS_OK:

				err_no	= msg->u.close.err_no;
				break;

			case OS_MSG_STS_MSG_SIZE:

				err_no	= EMSGSIZE;
				break;

			case OS_MSG_STS_MSG_TYPE:

				err_no	= EBADMSG;
				break;

			case OS_MSG_STS_MEM_MAP:

				err_no	= ENOMEM;
				break;

			case OS_MSG_STS_THREAD:

				err_no	= ENOEXEC;
				break;
		}

		break;
	}

	if (msg)
		os_msg_free(&msg);

	os_mbox_free(&mbox);

	if (sem == 0)
		os_sem_destroy(&fp->sem);

	if (fp)
	{
		memset(fp, 0, sizeof(os_file_t));
		free(fp);
	}

	return(err_no);
}


