// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/lib/rx.c $
// $Rev: 23013 $
// $Date: 2013-08-27 14:18:12 -0500 (Tue, 27 Aug 2013) $

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_sync_rx_get
*
*	Purpose:
*
*		Retrieve the current Rx specific SYNC configuration settings.
*
*	Arguments:
*
*		fd		The file rescriptor for the SIO4 to access.
*
*		rx		Record the settings here.
*
*	Returned:
*
*		0		All went well.
*		> 0		An appropriate error status.
*
******************************************************************************/

int sio4_sync_rx_get(int fd, sio4_sync_rx_t* rx)
{
	int	ret	= 0;
	int	sts;
	s32	set;

	// Rx Enable
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_ENABLE, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->enable	= set;

	// Rx Least Significant Bit First
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_BIT_ORDER, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->bit_order	= set;

	// Rx Count Reset
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_COUNT_ERROR, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->reset	= (set == SIO4_SYNC_RX_COUNT_ERROR_YES) ? 1 : 0;

	// Rx Word Size
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_BIT_COUNT, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->word_size	= set;

	// Rx Clock Configuration
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXC_CFG, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->clock.cfg	= set;

	// Rx Clock Polarity
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXC_POL, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->clock.pol	= set;

	// Rx Data Configuration
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXD_CFG, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->data.cfg	= set;

	// Rx Data Legacy
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_LEG_RXD_CFG, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->data.legacy	= set;

	// Rx Envelope Configuration
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXE_CFG, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->env.cfg	= set;

	// Rx Envelope Polarity
	set	= -1;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXE_POL, &set);
	ret	= (sts == -1) ? errno : ret;
	rx->env.pol	= set;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_sync_rx_set
*
*	Purpose:
*
*		Update the current Rx specific SYNC configuration.
*
*	Arguments:
*
*		fd		The file rescriptor for the SIO4 to access.
*
*		rx		The setting to apply.
*
*	Returned:
*
*		0		All went well.
*		> 0		An appropriate error status.
*
******************************************************************************/

int sio4_sync_rx_set(int fd, const sio4_sync_rx_t* rx)
{
	int	ret	= 0;
	int	sts;
	s32	set;

	// Rx Enable
	set	= rx->enable;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_ENABLE, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Least Significant Bit First

	set	= rx->bit_order;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_BIT_ORDER, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Count Reset

	if (rx->reset)
		set	= 1;
	else
		set	= 0;

	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RX_COUNT_RESET, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Word Size: this is a read only setting

	// Rx Clock Configuration
	set	= rx->clock.cfg;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXC_CFG, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Clock Polarity
	set	= rx->clock.pol;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXC_POL, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Data Configuration
	set	= rx->data.cfg;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXD_CFG, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Data Legacy
	set	= rx->data.legacy;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_LEG_RXD_CFG, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Envelope Configuration
	set	= rx->env.cfg;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXE_CFG, &set);
	ret	= (sts == -1) ? errno : ret;

	// Rx Envelope Polarity
	set	= rx->env.pol;
	sts	= sio4_ioctl(fd, SIO4_IOCTL_SYNC_RXE_POL, &set);
	ret	= (sts == -1) ? errno : ret;

	return(ret);
}


